# Scaling Large-Language-Model-based Multi-Agent Collaboration

<p align="center">
  <img src='./misc/macnet.png' width=550>
</p>

## 📖 Overview

MacNet focuses on supporting the research of agent scaling on arbitrary topologies using the Multi-Agent Collaboration Networks (MacNet) framework. MacNet utilizes directed acyclic graphs to facilitate effective task-oriented collaboration among agents through linguistic interactions. It supports cooperation across various topologies and among a large number of agents without exceeding context limits.

## ⚡️ Quickstart

To get started with MacNet, follow these steps:

  <p align="center">
    <img src='./misc/topo.png' width=500>
  </p>

1. **Generate Graph Structure:** Run `generate_graph.py` to generate the desired graph structure for your multi-agent collaboration network. You can customize the number of nodes and the topology of the graph. For example:

   ```
   python generate_graph.py --node_num 4 --topology tree 
   ```

2. **Configure MacNet:** The graph topologies can be edited manually. The graph structure is stored in the `graph` variable in `config.yaml`. `graph` is a list object with elements of type str, each element represents one or more edges. The parsing rules are as follows:    
    ```
    "->" represents an edge, e.g., "1->2" represents a directed edge pointing from node 1 to node 2. Use the hyphen "-" to represent consecutive nodes, e.g., "1-3->4" is equivalent to "'1->4','2->4','3->4'". "1-3->4-6" is a fully connected layer of three nodes per layer.
    ```
    Alternatively, run `generate_graph.py` to automatically update the graph config in the `config.yaml` file. You can check other parameters for deploying agents, including the LLM backend and prompt. Additionally, MacNet supports the **SRDD_Profile**, which is a large-scale agent role prompt collection. It allows you to randomly select system profile prompts for the deployed agents in the topology based on the software category. To use SRDD_Profile, specify the category using the `--type` parameter when running `run.py`.

  <p align="center">
    <img src='./misc/agent_deploy.png' width=500>
  </p>

3. **Run MacNet:** Execute `run.py` to deploy agents on the specified graph topology and generate software based on the configuration in `config.yaml`. During this process, you can observe the complete graph structure, the code changes occurring from one node to another, and the corresponding agent suggestions. MacNet will run agents on the graph in the topological order. You can observe the code changes in each node transition and agent suggestions on each edge in real-time.


   ```
   python run.py --task "Develop a basic Gomoku game." --name "Gomoku" [--type <software_category>]
   ```

   - `--task`: Specifies the prompt or description of the software to be developed.
   - `--name`: Specifies the name of the software project.
   - `--type` (optional): Specifies the software category for selecting system profile prompts from the SRDD_Profile dataset.

  <p align="center">
    <img src='./misc/topo_order.png' width=500>
  </p>
  

4. **Access Generated Software:** After the execution of `run.py`, the generated software can be found in the `WareHouse` directory. Each software project will have its own folder with a timestamp.

5. **View Logs:** The complete logs of the MacNet execution can be found in the `MacNetLog` directory. The logs are organized by timestamp and provide detailed information about the multi-agent collaboration process.

## 📝 Example
- Example of various generated topologies:

<p align="center">
  <img src='./misc/topo_example2.png' width=550>
</p>

<p align="center">
  <img src='./misc/topo_example.png' width=500>
</p>

- Example of the MacNet running process, including the suggestions and code diffs between agents on adjacent nodes:
<p align="center">
  <img src='./misc/running_example.png' width=500>
</p>

- Example of the generated software:
<p align="center">
  <img src='./misc/software_example.png' width=400>
</p>


## 📚 Datasets
### **MMLU**
MMLU (Massive Multitask Language Understanding) is a massive multitask test consisting of multiple-choice questions from various branches of knowledge. The test spans subjects in the humanities, social sciences, hard sciences, and other areas that are important for some people to learn. There are 57 tasks in total.
 
Dan Hendrycks, Collin Burns, Steven Basart, Andy Zou, Mantas Mazeika, Dawn Song, et al. 2020. Measuring Massive Multitask Language Understanding. In *arXiv:2009.03300*. <https://doi.org/10.48550/arXiv.2009.03300>

### **HumanEval**
HumanEval is a set of 164 hand-written programming problems. Each problem includes a function signature, docstring, body, and several unit tests, with an average of 7.7 tests per problem. Programming tasks in the HumanEval dataset assess language comprehension, reasoning, algorithms, and simple mathematics.

Mark Chen, Jerry Tworek, Heewoo Jun, Qiming Yuan, Henrique Ponde de Oliveira Pinto, Jared Kaplan, et al. 2021. Evaluating Large Language Models Trained on Code. In *arXiv:2107.03374*. <https://doi.org/10.48550/arXiv.2107.03374>

### **SRDD**
SRDD (Software Requirement Description Dataset) includes important software categories from popular platforms such as Ubuntu, Google Play, Microsoft Store, and Apple Store. It comprises 1,200 software task prompts that have been carefully categorized into 5 main areas: Education, Work, Life, Game, and Creation. All these areas are further divided into 40 subcategories, and each subcategory contains 30 unique task prompts.    

Chen Qian, Wei Liu, Hongzhang Liu, Nuo Chen, Yufan Dang, Jiahao Li, et al. 2024. ChatDev: Communicative Agents for Software Development. In *The Annual Meeting of the Association for Computational Linguistics (ACL)*. <https://aclanthology.org/2024.acl-long.810/>
### **CommonGen** 
CommonGen is a novel constrained generation task that requires machines to generate a sentence describing a day-to-day scene using concepts from a given concept-set. For example, given a set of concepts: {dog, frisbee, catch, throw}, machines are required to generate a sentence such as "a man **throws** a **frisbee** and his **dog** **catches** it in the air".

Bill Yuchen Lin, Wangchunshu Zhou, Ming Shen, Pei Zhou, Chandra Bhagavatula, Yejin Choi, Xiang Ren. 2019. CommonGen: A Constrained Text Generation Challenge for Generative Commonsense Reasoning. In *arXiv:1911.03705*. <https://doi.org/10.48550/arXiv.1911.03705> 
